package com.sun.corba.se.impl.encoding;

import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.UnmappableCharacterException;
import java.nio.charset.MalformedInputException;
import java.nio.CharBuffer;
import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.Charset;
import java.nio.ByteBuffer;
import java.nio.charset.CharsetEncoder;
import com.sun.corba.se.impl.logging.OMGSystemException;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;

public class CodeSetConversion
{
    private static CodeSetConversion implementation;
    private static final int FALLBACK_CODESET = 0;
    private CodeSetCache cache;
    
    public CodeSetConversion.CTBConverter getCTBConverter(OSFCodeSetRegistry.Entry entry) {
        return new CodeSetConversion.JavaCTBConverter(entry, entry.isFixedWidth() ? entry.getMaxBytesPerChar() : 1);
    }
    
    public CodeSetConversion.CTBConverter getCTBConverter(OSFCodeSetRegistry.Entry entry, boolean b, boolean b2) {
        if (entry == OSFCodeSetRegistry.UCS_2) {
            return new CodeSetConversion.UTF16CTBConverter(b);
        }
        if (entry == OSFCodeSetRegistry.UTF_16) {
            if (b2) {
                return new CodeSetConversion.UTF16CTBConverter();
            }
            return new CodeSetConversion.UTF16CTBConverter(b);
        }
        else {
            return new CodeSetConversion.JavaCTBConverter(entry, entry.isFixedWidth() ? entry.getMaxBytesPerChar() : 1);
        }
    }
    
    public CodeSetConversion.BTCConverter getBTCConverter(OSFCodeSetRegistry.Entry entry) {
        return new CodeSetConversion.JavaBTCConverter(entry);
    }
    
    public CodeSetConversion.BTCConverter getBTCConverter(OSFCodeSetRegistry.Entry entry, boolean b) {
        if (entry == OSFCodeSetRegistry.UTF_16 || entry == OSFCodeSetRegistry.UCS_2) {
            return new CodeSetConversion.UTF16BTCConverter(b);
        }
        return new CodeSetConversion.JavaBTCConverter(entry);
    }
    
    private int selectEncoding(CodeSetComponentInfo.CodeSetComponent codeSetComponent, CodeSetComponentInfo.CodeSetComponent codeSetComponent2) {
        int nativeCodeSet = codeSetComponent2.nativeCodeSet;
        if (nativeCodeSet == 0) {
            if (codeSetComponent2.conversionCodeSets.length > 0) {
                nativeCodeSet = codeSetComponent2.conversionCodeSets[0];
            }
            else {
                return 0;
            }
        }
        if (codeSetComponent.nativeCodeSet == nativeCodeSet) {
            return nativeCodeSet;
        }
        int i = 0;
        while (i < codeSetComponent.conversionCodeSets.length) {
            if (nativeCodeSet == codeSetComponent.conversionCodeSets[i]) {
                return nativeCodeSet;
            }
            ++i;
        }
        int j = 0;
        while (j < codeSetComponent2.conversionCodeSets.length) {
            if (codeSetComponent.nativeCodeSet == codeSetComponent2.conversionCodeSets[j]) {
                return codeSetComponent.nativeCodeSet;
            }
            ++j;
        }
        for (int k = 0; k < codeSetComponent2.conversionCodeSets.length; ++k) {
            int l = 0;
            while (l < codeSetComponent.conversionCodeSets.length) {
                if (codeSetComponent2.conversionCodeSets[k] == codeSetComponent.conversionCodeSets[l]) {
                    return codeSetComponent2.conversionCodeSets[k];
                }
                ++l;
            }
        }
        return 0;
    }
    
    public CodeSetComponentInfo.CodeSetContext negotiate(CodeSetComponentInfo codeSetComponentInfo, CodeSetComponentInfo codeSetComponentInfo2) {
        int n = this.selectEncoding(codeSetComponentInfo.getCharComponent(), codeSetComponentInfo2.getCharComponent());
        if (n == 0) {
            n = OSFCodeSetRegistry.UTF_8.getNumber();
        }
        int n2 = this.selectEncoding(codeSetComponentInfo.getWCharComponent(), codeSetComponentInfo2.getWCharComponent());
        if (n2 == 0) {
            n2 = OSFCodeSetRegistry.UTF_16.getNumber();
        }
        return new CodeSetComponentInfo.CodeSetContext(n, n2);
    }
    
    private CodeSetConversion() {
        this.cache = new CodeSetCache();
    }
    
    public static final CodeSetConversion impl() {
        return CodeSetConversion.CodeSetConversionHolder.csc;
    }
    
    CodeSetConversion(CodeSetConversion.1 object) {
        this();
    }
    
    public abstract static class CTBConverter
    {
        public abstract void convert(char p0);
        
        public abstract void convert(String p0);
        
        public abstract int getNumBytes();
        
        public abstract float getMaxBytesPerChar();
        
        public abstract boolean isFixedWidthEncoding();
        
        public abstract int getAlignment();
        
        public abstract byte[] getBytes();
    }
    
    public abstract static class BTCConverter
    {
        public abstract boolean isFixedWidthEncoding();
        
        public abstract int getFixedCharWidth();
        
        public abstract int getNumChars();
        
        public abstract char[] getChars(byte[] p0, int p1, int p2);
    }
    
    private class JavaCTBConverter extends CodeSetConversion.CTBConverter
    {
        private ORBUtilSystemException wrapper;
        private OMGSystemException omgWrapper;
        private CharsetEncoder ctb;
        private int alignment;
        private char[] chars;
        private int numBytes;
        private int numChars;
        private ByteBuffer buffer;
        private OSFCodeSetRegistry.Entry codeset;
        
        public JavaCTBConverter(OSFCodeSetRegistry.Entry codeset, int alignment) {
            this.wrapper = ORBUtilSystemException.get("rpc.encoding");
            this.omgWrapper = OMGSystemException.get("rpc.encoding");
            this.chars = null;
            this.numBytes = 0;
            this.numChars = 0;
            try {
                this.ctb = CodeSetConversion.this.cache.getCharToByteConverter(codeset.getName());
                if (this.ctb == null) {
                    this.ctb = Charset.forName(codeset.getName()).newEncoder();
                    CodeSetConversion.this.cache.setConverter(codeset.getName(), this.ctb);
                }
            }
            catch (IllegalCharsetNameException ex) {
                throw this.wrapper.invalidCtbConverterName(ex, codeset.getName());
            }
            catch (UnsupportedCharsetException ex2) {
                throw this.wrapper.invalidCtbConverterName(ex2, codeset.getName());
            }
            this.codeset = codeset;
            this.alignment = alignment;
        }
        
        @Override
        public final float getMaxBytesPerChar() {
            return this.ctb.maxBytesPerChar();
        }
        
        @Override
        public void convert(char c) {
            if (this.chars == null) {
                this.chars = new char[1];
            }
            this.chars[0] = c;
            this.numChars = 1;
            this.convertCharArray();
        }
        
        @Override
        public void convert(String s) {
            if (this.chars == null || this.chars.length < s.length()) {
                this.chars = new char[s.length()];
            }
            s.getChars(0, this.numChars = s.length(), this.chars, 0);
            this.convertCharArray();
        }
        
        @Override
        public final int getNumBytes() {
            return this.numBytes;
        }
        
        @Override
        public final int getAlignment() {
            return this.alignment;
        }
        
        @Override
        public final boolean isFixedWidthEncoding() {
            return this.codeset.isFixedWidth();
        }
        
        @Override
        public byte[] getBytes() {
            return this.buffer.array();
        }
        
        private void convertCharArray() {
            try {
                this.buffer = this.ctb.encode(CharBuffer.wrap(this.chars, 0, this.numChars));
                this.numBytes = this.buffer.limit();
            }
            catch (IllegalStateException ex) {
                throw this.wrapper.ctbConverterFailure(ex);
            }
            catch (MalformedInputException ex2) {
                throw this.wrapper.badUnicodePair(ex2);
            }
            catch (UnmappableCharacterException ex3) {
                throw this.omgWrapper.charNotInCodeset(ex3);
            }
            catch (CharacterCodingException ex4) {
                throw this.wrapper.ctbConverterFailure(ex4);
            }
        }
    }
    
    private class UTF16CTBConverter extends CodeSetConversion.JavaCTBConverter
    {
        public UTF16CTBConverter() {
            super(OSFCodeSetRegistry.UTF_16, 2);
        }
        
        public UTF16CTBConverter(boolean b) {
            super(b ? OSFCodeSetRegistry.UTF_16LE : OSFCodeSetRegistry.UTF_16BE, 2);
        }
    }
    
    private class JavaBTCConverter extends CodeSetConversion.BTCConverter
    {
        private ORBUtilSystemException wrapper;
        private OMGSystemException omgWrapper;
        protected CharsetDecoder btc;
        private char[] buffer;
        private int resultingNumChars;
        private OSFCodeSetRegistry.Entry codeset;
        
        public JavaBTCConverter(OSFCodeSetRegistry.Entry codeset) {
            this.wrapper = ORBUtilSystemException.get("rpc.encoding");
            this.omgWrapper = OMGSystemException.get("rpc.encoding");
            this.btc = this.getConverter(codeset.getName());
            this.codeset = codeset;
        }
        
        @Override
        public final boolean isFixedWidthEncoding() {
            return this.codeset.isFixedWidth();
        }
        
        @Override
        public final int getFixedCharWidth() {
            return this.codeset.getMaxBytesPerChar();
        }
        
        @Override
        public final int getNumChars() {
            return this.resultingNumChars;
        }
        
        @Override
        public char[] getChars(byte[] array, int offset, int length) {
            try {
                CharBuffer decode = this.btc.decode(ByteBuffer.wrap(array, offset, length));
                this.resultingNumChars = decode.limit();
                if (decode.limit() == decode.capacity()) {
                    this.buffer = decode.array();
                }
                else {
                    this.buffer = new char[decode.limit()];
                    decode.get(this.buffer, 0, decode.limit()).position(0);
                }
                return this.buffer;
            }
            catch (IllegalStateException ex) {
                throw this.wrapper.btcConverterFailure(ex);
            }
            catch (MalformedInputException ex2) {
                throw this.wrapper.badUnicodePair(ex2);
            }
            catch (UnmappableCharacterException ex3) {
                throw this.omgWrapper.charNotInCodeset(ex3);
            }
            catch (CharacterCodingException ex4) {
                throw this.wrapper.btcConverterFailure(ex4);
            }
        }
        
        protected CharsetDecoder getConverter(String charsetName) {
            CharsetDecoder charsetDecoder;
            try {
                charsetDecoder = CodeSetConversion.this.cache.getByteToCharConverter(charsetName);
                if (charsetDecoder == null) {
                    charsetDecoder = Charset.forName(charsetName).newDecoder();
                    CodeSetConversion.this.cache.setConverter(charsetName, charsetDecoder);
                }
            }
            catch (IllegalCharsetNameException ex) {
                throw this.wrapper.invalidBtcConverterName(ex, charsetName);
            }
            return charsetDecoder;
        }
    }
    
    private class UTF16BTCConverter extends CodeSetConversion.JavaBTCConverter
    {
        private boolean defaultToLittleEndian;
        private boolean converterUsesBOM;
        private static final char UTF16_BE_MARKER = '\ufeff';
        private static final char UTF16_LE_MARKER = '\ufffe';
        
        public UTF16BTCConverter(boolean defaultToLittleEndian) {
            super(OSFCodeSetRegistry.UTF_16);
            this.converterUsesBOM = true;
            this.defaultToLittleEndian = defaultToLittleEndian;
        }
        
        @Override
        public char[] getChars(byte[] array, int n, int n2) {
            if (this.hasUTF16ByteOrderMarker(array, n, n2)) {
                if (!this.converterUsesBOM) {
                    this.switchToConverter(OSFCodeSetRegistry.UTF_16);
                }
                this.converterUsesBOM = true;
                return super.getChars(array, n, n2);
            }
            else {
                if (this.converterUsesBOM) {
                    if (this.defaultToLittleEndian) {
                        this.switchToConverter(OSFCodeSetRegistry.UTF_16LE);
                    }
                    else {
                        this.switchToConverter(OSFCodeSetRegistry.UTF_16BE);
                    }
                    this.converterUsesBOM = false;
                }
                return super.getChars(array, n, n2);
            }
        }
        
        private boolean hasUTF16ByteOrderMarker(byte[] array, int n, int n2) {
            if (n2 >= 4) {
                int n3 = (char)((array[n] & 0xFF) << 8 | (array[n + 1] & 0xFF) << 0);
                return n3 == '\ufeff' || n3 == '\ufffe';
            }
            return false;
        }
        
        private void switchToConverter(OSFCodeSetRegistry.Entry entry) {
            this.btc = super.getConverter(entry.getName());
        }
    }
    
    private static class CodeSetConversionHolder
    {
        static final CodeSetConversion csc;
        
        static {
            csc = new CodeSetConversion();
        }
    }
}
